@@ -46,18 +46,24 @@ def allNaN_arg(da, dim, stat):
     -------
     xarray.DataArray
     """
-    # generate a mask where entire axis along dimension is NaN
+    # Generate a mask where the entire axis along dimension is NaN.
     mask = da.isnull().all(dim)
 
     if stat == "max":
-        y = da.fillna(float(da.min() - 1))
-        y = y.argmax(dim=dim, skipna=True).where(~mask)
-        return y
+        # Generate the fill value for the all-NaN pixels. 
+        fill_value = float(da.min() - 1)
+        # Fill the all-NaN pixels.
+        y = xr.where(mask, fill_value, da)
+        y = y.argmax(dim=dim, skipna=True)
+        return mask, y
 
     if stat == "min":
-        y = da.fillna(float(da.max() + 1))
-        y = y.argmin(dim=dim, skipna=True).where(~mask)
-        return y
+        # Generate the fill value for the all-NaN pixels. 
+        fill_value = float(da.max() + 1)
+        # Fill the all-NaN pixels.
+        y = xr.where(mask, fill_value, da)
+        y = y.argmin(dim=dim, skipna=True)
+        return mask, y
 
 
 def _vpos(da):
@@ -69,9 +75,9 @@ def _vpos(da):
 
 def _pos(da):
     """
-    POS = DOY of peak of season
+    POS = Date of peak of season
     """
-    return da.isel(time=da.argmax("time")).time.dt.dayofyear
+    return da.isel(time=da.argmax("time")).time
 
 
 def _trough(da):
@@ -116,21 +122,23 @@ def _vsos(da, pos, method_sos="first"):
     distance = pos_greenup - median
 
     if method_sos == "first":
-        # find index (argmin) where distance is most negative
-        idx = allNaN_arg(distance, "time", "min").astype("int16")
+        # Find the time index (argmin) where the distance of values from the median is most negative.
+        mask, idx = allNaN_arg(distance, "time", "min")
+        idx = idx.astype("int16")
 
     if method_sos == "median":
-        # find index (argmin) where distance is smallest absolute value
-        idx = allNaN_arg(xr.ufuncs.fabs(distance), "time", "min").astype("int16")
-
-    return pos_greenup.isel(time=idx)
+        # Find the time index (argmin) where the distance of values from median is the smallest absolute value.
+        mask, idx = allNaN_arg(np.fabs(distance), "time", "min")
+        idx = idx.astype("int16")
+                         
+    return pos_greenup.isel(time=idx).where(~mask) # mask the all-NaN pixels 
 
 
 def _sos(vsos):
     """
-    SOS = DOY for start of season
+    SOS = Date for start of season
     """
-    return vsos.time.dt.dayofyear
+    return vsos.time
 
 
 def _veos(da, pos, method_eos="last"):
@@ -146,7 +154,7 @@ def _veos(da, pos, method_eos="last"):
         of the negative slopes on the senescing
         side of the curve.
     """
-    # select timesteps before peak of season (AKA greening)
+    # select timesteps after peak of season (AKA greening)
     senesce = da.where(da.time > pos.time)
     # find the first order slopes
     senesce_deriv = senesce.differentiate("time")
@@ -160,35 +168,36 @@ def _veos(da, pos, method_eos="last"):
     distance = neg_senesce - median
 
     if method_eos == "last":
-        # index where last negative slope occurs
-        idx = allNaN_arg(distance, "time", "min").astype("int16")
+        # Find the time index (argmin) where the last negative slope occurs.
+        mask, idx = allNaN_arg(distance, "time", "min")
+        idx = idx.astype("int16")
 
     if method_eos == "median":
-        # index where median occurs
-        idx = allNaN_arg(xr.ufuncs.fabs(distance), "time", "min").astype("int16")
+        # Find the time index (argmin) where the median occurs.
+        mask, idx = allNaN_arg(np.fabs(distance), "time", "min")
+        idx = idx.astype("int16")
 
-    return neg_senesce.isel(time=idx)
+    return neg_senesce.isel(time=idx).where(~mask) # mask the all-NaN pixels 
 
 
 def _eos(veos):
     """
-    EOS = DOY for end of seasonn
+    EOS = Date for end of season
     """
-    return veos.time.dt.dayofyear
+    return veos.time
 
 
 def _los(da, eos, sos):
     """
-    LOS = Length of season (in DOY)
+    LOS = Length of season (Days)
     """
-    los = eos - sos
+    los = (eos - sos).dt.days
     #handle negative values
     los = xr.where(
         los >= 0,
         los,
-        da.time.dt.dayofyear.values[-1] + (eos.where(los < 0) - sos.where(los < 0)),
+        (da.time.dt.dayofyear.values[-1] + (eos.where(los < 0) - sos.where(los < 0))).dt.days
     )
-
     return los
 
 
@@ -196,14 +205,14 @@ def _rog(vpos, vsos, pos, sos):
     """
     ROG = Rate of Greening (Days)
     """
-    return (vpos - vsos) / (pos - sos)
+    return (vpos - vsos) / (pos - sos).dt.days
 
 
 def _ros(veos, vpos, eos, pos):
     """
     ROG = Rate of Senescing (Days)
     """
-    return (veos - vpos) / (eos - pos)
+    return (veos - vpos) / (eos - pos).dt.days
 
 
 def xr_phenology(
@@ -230,7 +239,7 @@ def xr_phenology(
     xarray.DataArray containing a timeseries of a
     vegetation index like NDVI.
 
-    last modified June 2020
+    last modified June 2023
 
     Parameters
     ----------
@@ -243,14 +252,14 @@ def xr_phenology(
         due to inter-dependencies between metrics.
         Options include:
 
-        * `SOS` = DOY of start of season
-        * `POS` = DOY of peak of season
-        * `EOS` = DOY of end of season
+        * `SOS` = Date of start of season
+        * `POS` = Date of peak of season
+        * `EOS` = Date of end of season
         * `vSOS` = Value at start of season
         * `vPOS` = Value at peak of season
         * `vEOS` = Value at end of season
         * `Trough` = Minimum value of season
-        * `LOS` = Length of season (DOY)
+        * `LOS` = Length of season (Days)
         * `AOS` = Amplitude of season (in value units)
         * `ROG` = Rate of greening
         * `ROS` = Rate of senescence
@@ -281,9 +290,9 @@ def xr_phenology(
                 + "run da.compute() before passing dataArray."
             )
         stats_dtype = {
-            "SOS": np.int16,
-            "POS": np.int16,
-            "EOS": np.int16,
+            "SOS": np.dtype('<M8[ns]'),
+            "POS": np.dtype('<M8[ns]'),
+            "EOS": np.dtype('<M8[ns]'),
             "Trough": np.float32,
             "vSOS": np.float32,
             "vPOS": np.float32,
@@ -336,10 +345,16 @@ def xr_phenology(
     except:
         pass
 
-    # remove any remaining all-NaN pixels
-    mask = da.isnull().all("time")
-    da = da.where(~mask, other=0)
-
+    # Deal with any all-NaN pixels by filling with 0's.
+    da_all_nan_mask = da.isnull().all("time")
+    da = da.where(~da_all_nan_mask, other=0)
+    
+    # Create a template to use when handling errors calculating statistics.
+    xr_template = xr.full_like(da.isel(time=0).drop("time"), fill_value=np.nan, dtype=np.float32)
+    # Create time coordinates for the template. 
+    time_coords = xr.full_like(da.isel(time=0).drop("time"), fill_value=np.nan, dtype="datetime64[ns]")
+    time_coords.attrs = {'units': 'seconds since 1970-01-01 00:00:00'}
+    
     # calculate the statistics
     if verbose:
         print("      Phenology...")
@@ -347,9 +362,15 @@ def xr_phenology(
     pos = _pos(da)
     trough = _trough(da)
     aos = _aos(vpos, trough)
-    vsos = _vsos(da, pos, method_sos=method_sos)
+    try:
+        vsos = _vsos(da, pos, method_sos=method_sos)
+    except:
+        vsos = xr_template.assign_coords(time=time_coords)
     sos = _sos(vsos)
-    veos = _veos(da, pos, method_eos=method_eos)
+    try:
+        veos = _veos(da, pos, method_eos=method_eos)
+    except:
+        veos = xr_template.assign_coords(time=time_coords)
     eos = _eos(veos)
     los = _los(da, eos, sos)
     rog = _rog(vpos, vsos, pos, sos)
@@ -357,12 +378,12 @@ def xr_phenology(
 
     # Dictionary containing the statistics
     stats_dict = {
-        "SOS": sos.astype(np.int16),
-        "EOS": eos.astype(np.int16),
+        "SOS": sos.astype(np.dtype('<M8[ns]')),
+        "EOS": eos.astype(np.dtype('<M8[ns]')),
         "vSOS": vsos.astype(np.float32),
         "vPOS": vpos.astype(np.float32),
         "Trough": trough.astype(np.float32),
-        "POS": pos.astype(np.int16),
+        "POS": pos.astype(np.dtype('<M8[ns]')),
         "vEOS": veos.astype(np.float32),
         "LOS": los.astype(np.int16),
         "AOS": aos.astype(np.float32),
@@ -379,7 +400,11 @@ def xr_phenology(
             print("         " + stat)
         stats_keep = stats_dict.get(stat)
         ds[stat] = stats_dict[stat]
-
+    
+    # Set original all-NaN pixels back to NaN.
+    ds = ds.where(~da_all_nan_mask)
+    
+    # Try add back the crs.
     try:
         ds = assign_crs(ds, str(crs))
     except:
@@ -395,7 +420,7 @@ def temporal_statistics(da, stats):
     This function uses the hdstats temporal library:
     https://github.com/daleroberts/hdstats/blob/master/hdstats/ts.pyx
 
-    last modified June 2020
+    last modified June 2023
 
     Parameters
     ----------
@@ -489,9 +514,9 @@ def temporal_statistics(da, stats):
     # grab all the attributes of the xarray
     x, y, time, attrs = da.x, da.y, da.time, da.attrs
 
-    # deal with any all-NaN pixels by filling with 0's
-    mask = da.isnull().all("time")
-    da = da.where(~mask, other=0)
+    # Deal with any all-NaN pixels by filling with 0's.
+    da_all_nan_mask = da.isnull().all("time")
+    da = da.where(~da_all_nan_mask, other=0)
 
     # ensure dim order is correct for functions
     da = da.transpose("y", "x", "time").values
@@ -565,6 +590,9 @@ def temporal_statistics(da, stats):
             ds[stat] = xr.DataArray(
                 stat_func(da), attrs=attrs, coords={"x": x, "y": y}, dims=["y", "x"]
             )
+    
+    # Set original all-NaN pixels back to NaN.
+    ds = ds.where(~da_all_nan_mask)
 
     # try to add back the geobox
     try:
